@@ -1,8 +1,6 @@
 module ActionMailer
   module Helpers #:nodoc:
-    def self.append_features(base) #:nodoc:
-      super
-
+    def self.included(base) #:nodoc:
       # Initialize the base module to aggregate its helpers.
       base.class_inheritable_accessor :master_helper_module
       base.master_helper_module = Module.new
@@ -13,14 +11,12 @@ module ActionMailer
       base.class_eval do
         # Wrap inherited to create a new master helper module for subclasses.
         class << self
-          alias_method :inherited_without_helper, :inherited
-          alias_method :inherited, :inherited_with_helper
+          alias_method_chain :inherited, :helper
         end
 
         # Wrap initialize_template_class to extend new template class
         # instances with the master helper module.
-        alias_method :initialize_template_class_without_helper, :initialize_template_class
-        alias_method :initialize_template_class, :initialize_template_class_with_helper
+        alias_method_chain :initialize_template_class, :helper
       end
     end
 
@@ -53,7 +49,7 @@ module ActionMailer
               begin
                 require_dependency(file_name)
               rescue LoadError => load_error
-                requiree = / -- (.*?)(\.rb)?$/.match(load_error).to_a[1]
+                requiree = / -- (.*?)(\.rb)?$/.match(load_error.message).to_a[1]
                 msg = (requiree == file_name) ? "Missing helper file helpers/#{file_name}.rb" : "Can't load file: #{requiree}"
                 raise LoadError.new(msg).copy_blame!(load_error)
               end
@@ -76,7 +72,7 @@ module ActionMailer
         methods.flatten.each do |method|
           master_helper_module.module_eval <<-end_eval
             def #{method}(*args, &block)
-              controller.send(%(#{method}), *args, &block)
+              controller.send!(%(#{method}), *args, &block)
             end
           end_eval
         end
@@ -96,7 +92,7 @@ module ActionMailer
           inherited_without_helper(child)
           begin
             child.master_helper_module = Module.new
-            child.master_helper_module.send :include, master_helper_module
+            child.master_helper_module.send! :include, master_helper_module
             child.helper child.name.underscore
           rescue MissingSourceFile => e
             raise unless e.is_missing?("helpers/#{child.name.underscore}_helper")
@@ -112,4 +108,4 @@ module ActionMailer
         end
       end
   end
-end
\ No newline at end of file
+end
