@@ -1,4 +1,6 @@
 require 'fileutils'
+require 'uri'
+require 'set'
 
 module ActionController #:nodoc:
   # Caching is a cheap way of speeding up slow applications by keeping the result of calculations, renderings, and database calls
@@ -9,9 +11,13 @@ module ActionController #:nodoc:
   # Note: To turn off all caching and sweeping, set Base.perform_caching = false.
   module Caching
     def self.included(base) #:nodoc:
-      base.send(:include, Pages, Actions, Fragments, Sweeping)
-
       base.class_eval do
+        include Pages, Actions, Fragments
+
+        if defined? ActiveRecord
+          include Sweeping, SqlCache
+        end
+
         @@perform_caching = true
         cattr_accessor :perform_caching
       end
@@ -75,7 +81,7 @@ module ActionController #:nodoc:
           return unless perform_caching
 
           benchmark "Expired page: #{page_cache_file(path)}" do
-            File.delete(page_cache_path(path)) if File.exists?(page_cache_path(path))
+            File.delete(page_cache_path(path)) if File.exist?(page_cache_path(path))
           end
         end
 
@@ -94,14 +100,13 @@ module ActionController #:nodoc:
         # matches the triggering url.
         def caches_page(*actions)
           return unless perform_caching
-          actions.each do |action|
-            class_eval "after_filter { |c| c.cache_page if c.action_name == '#{action}' }"
-          end
+          actions = actions.map(&:to_s)
+          after_filter { |c| c.cache_page if actions.include?(c.action_name) }
         end
 
         private
           def page_cache_file(path)
-            name = ((path.empty? || path == "/") ? "/index" : URI.unescape(path))
+            name = (path.empty? || path == "/") ? "/index" : URI.unescape(path.chomp('/'))
             name << page_cache_extension unless (name.split('/').last || name).include? '.'
             return name
           end
@@ -115,26 +120,41 @@ module ActionController #:nodoc:
       #   expire_page :controller => "lists", :action => "show"
       def expire_page(options = {})
         return unless perform_caching
-        if options[:action].is_a?(Array)
-          options[:action].dup.each do |action|
-            self.class.expire_page(url_for(options.merge({ :only_path => true, :skip_relative_url_root => true, :action => action })))
+
+        if options.is_a?(Hash)
+          if options[:action].is_a?(Array)
+            options[:action].dup.each do |action|
+              self.class.expire_page(url_for(options.merge(:only_path => true, :skip_relative_url_root => true, :action => action)))
+            end
+          else
+            self.class.expire_page(url_for(options.merge(:only_path => true, :skip_relative_url_root => true)))
           end
         else
-          self.class.expire_page(url_for(options.merge({ :only_path => true, :skip_relative_url_root => true })))
+          self.class.expire_page(options)
         end
       end
 
-      # Manually cache the +content+ in the key determined by +options+. If no content is provided, the contents of @response.body is used
-      # If no options are provided, the current +options+ for this action is used. Example:
+      # Manually cache the +content+ in the key determined by +options+. If no content is provided, the contents of response.body is used
+      # If no options are provided, the requested url is used. Example:
       #   cache_page "I'm the cached content", :controller => "lists", :action => "show"
-      def cache_page(content = nil, options = {})
+      def cache_page(content = nil, options = nil)
         return unless perform_caching && caching_allowed
-        self.class.cache_page(content || @response.body, url_for(options.merge({ :only_path => true, :skip_relative_url_root => true })))
+
+        path = case options
+          when Hash
+            url_for(options.merge(:only_path => true, :skip_relative_url_root => true, :format => params[:format]))
+          when String
+            options
+          else
+            request.path
+        end
+
+        self.class.cache_page(content || response.body, path)
       end
 
       private
         def caching_allowed
-          !@request.post? && @response.headers['Status'] && @response.headers['Status'].to_i < 400
+          request.get? && response.headers['Status'].to_i == 200
         end
     end
 
@@ -155,49 +175,125 @@ module ActionController #:nodoc:
     # the current host and the path. So a page that is accessed at http://david.somewhere.com/lists/show/1 will result in a fragment named
     # "david.somewhere.com/lists/show/1". This allows the cacher to differentiate between "david.somewhere.com/lists/" and
     # "jamis.somewhere.com/lists/" -- which is a helpful way of assisting the subdomain-as-account-key pattern.
+    #
+    # Different representations of the same resource, e.g. <tt>http://david.somewhere.com/lists</tt> and <tt>http://david.somewhere.com/lists.xml</tt>
+    # are treated like separate requests and so are cached separately. Keep in mind when expiring an action cache that <tt>:action => 'lists'</tt> is not the same
+    # as <tt>:action => 'list', :format => :xml</tt>.
+    #
+    # You can set modify the default action cache path by passing a :cache_path option.  This will be passed directly to ActionCachePath.path_for.  This is handy
+    # for actions with multiple possible routes that should be cached differently.  If a block is given, it is called with the current controller instance.
+    #
+    #   class ListsController < ApplicationController
+    #     before_filter :authenticate, :except => :public
+    #     caches_page   :public
+    #     caches_action :show, :cache_path => { :project => 1 }
+    #     caches_action :show, :cache_path => Proc.new { |controller| 
+    #       controller.params[:user_id] ? 
+    #         controller.send(:user_list_url, c.params[:user_id], c.params[:id]) :
+    #         controller.send(:list_url, c.params[:id]) }
+    #   end
     module Actions
-      def self.append_features(base) #:nodoc:
-        super
+      def self.included(base) #:nodoc:
         base.extend(ClassMethods)
-        base.send(:attr_accessor, :rendered_action_cache)
+          base.class_eval do
+            attr_accessor :rendered_action_cache, :action_cache_path
+            alias_method_chain :protected_instance_variables, :action_caching
+          end
       end
 
-      module ClassMethods #:nodoc:
+      module ClassMethods
+        # Declares that +actions+ should be cached.
+        # See ActionController::Caching::Actions for details.
         def caches_action(*actions)
           return unless perform_caching
           around_filter(ActionCacheFilter.new(*actions))
         end
       end
 
+      def protected_instance_variables_with_action_caching
+        protected_instance_variables_without_action_caching + %w(@action_cache_path)
+      end
+
       def expire_action(options = {})
         return unless perform_caching
         if options[:action].is_a?(Array)
           options[:action].dup.each do |action|
-            expire_fragment(url_for(options.merge({ :action => action })).split("://").last)
+            expire_fragment(ActionCachePath.path_for(self, options.merge({ :action => action })))
           end
         else
-          expire_fragment(url_for(options).split("://").last)
+          expire_fragment(ActionCachePath.path_for(self, options))
         end
       end
 
       class ActionCacheFilter #:nodoc:
-        def initialize(*actions)
-          @actions = actions
+        def initialize(*actions, &block)
+          @options = actions.extract_options!
+          @actions = Set.new actions
         end
 
         def before(controller)
           return unless @actions.include?(controller.action_name.intern)
-          if cache = controller.read_fragment(controller.url_for.split("://").last)
+          cache_path = ActionCachePath.new(controller, path_options_for(controller, @options))
+          if cache = controller.read_fragment(cache_path.path)
             controller.rendered_action_cache = true
-            controller.send(:render_text, cache)
+            set_content_type!(controller, cache_path.extension)
+            controller.send!(:render_for_text, cache)
             false
+          else
+            controller.action_cache_path = cache_path
           end
         end
 
         def after(controller)
-          return if !@actions.include?(controller.action_name.intern) || controller.rendered_action_cache
-          controller.write_fragment(controller.url_for.split("://").last, controller.response.body)
+          return if !@actions.include?(controller.action_name.intern) || controller.rendered_action_cache || !caching_allowed(controller)
+          controller.write_fragment(controller.action_cache_path.path, controller.response.body)
         end
+
+        private
+          def set_content_type!(controller, extension)
+            controller.response.content_type = Mime::Type.lookup_by_extension(extension).to_s if extension
+          end
+
+          def path_options_for(controller, options)
+            ((path_options = options[:cache_path]).respond_to?(:call) ? path_options.call(controller) : path_options) || {}
+          end
+
+          def caching_allowed(controller)
+            controller.request.get? && controller.response.headers['Status'].to_i == 200
+          end
+      end
+      
+      class ActionCachePath
+        attr_reader :path, :extension
+        
+        class << self
+          def path_for(controller, options)
+            new(controller, options).path
+          end
+        end
+        
+        def initialize(controller, options = {})
+          @extension = extract_extension(controller.request.path)
+          path = controller.url_for(options).split('://').last
+          normalize!(path)
+          add_extension!(path, @extension)
+          @path = URI.unescape(path)
+        end
+        
+        private
+          def normalize!(path)
+            path << 'index' if path[-1] == ?/
+          end
+        
+          def add_extension!(path, extension)
+            path << ".#{extension}" if extension
+          end
+          
+          def extract_extension(file_path)
+            # Don't want just what comes after the last '.' to accommodate multi part extensions
+            # such as tar.gz.
+            file_path[/^[^.]+\.(.+)$/, 1]
+          end
       end
     end
 
@@ -208,28 +304,31 @@ module ActionController #:nodoc:
     #   <b>Hello <%= @name %></b>
     #   <% cache do %>
     #     All the topics in the system:
-    #     <%= render_collection_of_partials "topic", Topic.find_all %>
+    #     <%= render :partial => "topic", :collection => Topic.find(:all) %>
     #   <% end %>
     #
-    # This cache will bind to the name of action that called it. So you would be able to invalidate it using
-    # <tt>expire_fragment(:controller => "topics", :action => "list")</tt> -- if that was the controller/action used. This is not too helpful
-    # if you need to cache multiple fragments per action or if the action itself is cached using <tt>caches_action</tt>. So instead we should
-    # qualify the name of the action used with something like:
+    # This cache will bind to the name of the action that called it, so if this code was part of the view for the topics/list action, you would 
+    # be able to invalidate it using <tt>expire_fragment(:controller => "topics", :action => "list")</tt>. 
+    # 
+    # This default behavior is of limited use if you need to cache multiple fragments per action or if the action itself is cached using 
+    # <tt>caches_action</tt>, so we also have the option to qualify the name of the cached fragment with something like:
     #
     #   <% cache(:action => "list", :action_suffix => "all_topics") do %>
     #
-    # That would result in a name such as "/topics/list/all_topics", which wouldn't conflict with any action cache and neither with another
-    # fragment using a different suffix. Note that the URL doesn't have to really exist or be callable. We're just using the url_for system
-    # to generate unique cache names that we can refer to later for expirations. The expiration call for this example would be
-    # <tt>expire_fragment(:controller => "topics", :action => "list", :action_suffix => "all_topics")</tt>.
+    # That would result in a name such as "/topics/list/all_topics", avoiding conflicts with the action cache and with any fragments that use a 
+    # different suffix. Note that the URL doesn't have to really exist or be callable - the url_for system is just used to generate unique 
+    # cache names that we can refer to when we need to expire the cache. 
+    # 
+    # The expiration call for this example is:
+    # 
+    #   expire_fragment(:controller => "topics", :action => "list", :action_suffix => "all_topics")
     #
     # == Fragment stores
     #
-    # In order to use the fragment caching, you need to designate where the caches should be stored. This is done by assigning a fragment store
-    # of which there are four different kinds:
+    # By default, cached fragments are stored in memory. The available store options are:
     #
-    # * FileStore: Keeps the fragments on disk in the +cache_path+, which works well for all types of environments and shares the fragments for
-    #   all the web server processes running off the same application directory.
+    # * FileStore: Keeps the fragments on disk in the +cache_path+, which works well for all types of environments and allows all 
+    #   processes running from the same application directory to access the cached content.
     # * MemoryStore: Keeps the fragments in memory, which is fine for WEBrick and for FCGI (if you don't care that each FCGI process holds its
     #   own fragment store). It's not suitable for CGI as the process is thrown away at the end of each request. It can potentially also take
     #   up a lot of memory since each process keeps all the caches in memory.
@@ -246,12 +345,12 @@ module ActionController #:nodoc:
     #   ActionController::Base.fragment_cache_store = :mem_cache_store, "localhost"
     #   ActionController::Base.fragment_cache_store = MyOwnStore.new("parameter")
     module Fragments
-      def self.append_features(base) #:nodoc:
-        super
+      def self.included(base) #:nodoc:
         base.class_eval do
           @@fragment_cache_store = MemoryStore.new
           cattr_reader :fragment_cache_store
 
+          # Defines the storage option for cached fragments
           def self.fragment_cache_store=(store_option)
             store, *parameters = *([ store_option ].flatten)
             @@fragment_cache_store = if store.is_a?(Symbol)
@@ -265,6 +364,9 @@ module ActionController #:nodoc:
         end
       end
 
+      # Given a name (as described in <tt>expire_fragment</tt>), returns a key suitable for use in reading, 
+      # writing, or expiring a cached fragment. If the name is a hash, the generated name is the return
+      # value of url_for on that hash (without the protocol).
       def fragment_cache_key(name)
         name.is_a?(Hash) ? url_for(name).split("://").last : name
       end
@@ -273,7 +375,7 @@ module ActionController #:nodoc:
       def cache_erb_fragment(block, name = {}, options = nil)
         unless perform_caching then block.call; return end
 
-        buffer = eval("_erbout", block.binding)
+        buffer = eval(ActionView::Base.erb_variable, block.binding)
 
         if cache = read_fragment(name, options)
           buffer.concat(cache)
@@ -284,6 +386,7 @@ module ActionController #:nodoc:
         end
       end
 
+      # Writes <tt>content</tt> to the location signified by <tt>name</tt> (see <tt>expire_fragment</tt> for acceptable formats)
       def write_fragment(name, content, options = nil)
         return unless perform_caching
 
@@ -294,6 +397,7 @@ module ActionController #:nodoc:
         content
       end
 
+      # Reads a cached fragment from the location signified by <tt>name</tt> (see <tt>expire_fragment</tt> for acceptable formats)
       def read_fragment(name, options = nil)
         return unless perform_caching
 
@@ -306,7 +410,12 @@ module ActionController #:nodoc:
       # Name can take one of three forms:
       # * String: This would normally take the form of a path like "pages/45/notes"
       # * Hash: Is treated as an implicit call to url_for, like { :controller => "pages", :action => "notes", :id => 45 }
-      # * Regexp: Will destroy all the matched fragments, example: %r{pages/\d*/notes} Ensure you do not specify start and finish in the regex (^$) because the actual filename matched looks like ./cache/filename/path.cache
+      # * Regexp: Will destroy all the matched fragments, example:
+      #     %r{pages/\d*/notes}
+      #   Ensure you do not specify start and finish in the regex (^$) because
+      #   the actual filename matched looks like ./cache/filename/path.cache
+      #   Regexp expiration is only supported on caches that can iterate over
+      #   all keys (unlike memcached).
       def expire_fragment(name, options = nil)
         return unless perform_caching
 
@@ -323,11 +432,6 @@ module ActionController #:nodoc:
         end
       end
 
-      # Deprecated -- just call expire_fragment with a regular expression
-      def expire_matched_fragments(matcher = /.*/, options = nil) #:nodoc:
-        expire_fragment(matcher, options)
-      end
-
 
       class UnthreadedMemoryStore #:nodoc:
         def initialize #:nodoc:
@@ -374,7 +478,7 @@ module ActionController #:nodoc:
           super
           if ActionController::Base.allow_concurrency
             @mutex = Mutex.new
-            MemoryStore.send(:include, ThreadSafety)
+            MemoryStore.module_eval { include ThreadSafety }
           end
         end
       end
@@ -389,6 +493,8 @@ module ActionController #:nodoc:
         end
       end
 
+    begin
+      require_library_or_gem 'memcache'
       class MemCacheStore < MemoryStore #:nodoc:
         attr_reader :addresses
 
@@ -400,6 +506,9 @@ module ActionController #:nodoc:
           @data = MemCache.new(*addresses)
         end
       end
+    rescue LoadError
+      # MemCache wasn't available so neither can the store be
+    end
 
       class UnthreadedFileStore #:nodoc:
         attr_reader :cache_path
@@ -430,7 +539,7 @@ module ActionController #:nodoc:
             if f =~ matcher
               begin
                 File.delete(f)
-              rescue Object => e
+              rescue SystemCallError => e
                 # If there's no cache, then there's nothing to complain about
               end
             end
@@ -443,7 +552,7 @@ module ActionController #:nodoc:
           end
 
           def ensure_cache_path(path)
-            FileUtils.makedirs(path) unless File.exists?(path)
+            FileUtils.makedirs(path) unless File.exist?(path)
           end
 
           def search_dir(dir, &callback)
@@ -464,7 +573,7 @@ module ActionController #:nodoc:
             super(cache_path)
             if ActionController::Base.allow_concurrency
               @mutex = Mutex.new
-              FileStore.send(:include, ThreadSafety)
+              FileStore.module_eval { include ThreadSafety }
             end
           end
         end
@@ -493,18 +602,16 @@ module ActionController #:nodoc:
     #
     # In the example above, four actions are cached and three actions are responsible for expiring those caches.
     module Sweeping
-      def self.append_features(base) #:nodoc:
-        super
+      def self.included(base) #:nodoc:
         base.extend(ClassMethods)
       end
 
       module ClassMethods #:nodoc:
         def cache_sweeper(*sweepers)
           return unless perform_caching
-          configuration = sweepers.last.is_a?(Hash) ? sweepers.pop : {}
+          configuration = sweepers.extract_options!
           sweepers.each do |sweeper|
-            observer(sweeper)
-
+            ActiveRecord::Base.observers << sweeper if defined?(ActiveRecord) and defined?(ActiveRecord::Base)
             sweeper_instance = Object.const_get(Inflector.classify(sweeper)).instance
 
             if sweeper_instance.is_a?(Sweeper)
@@ -521,10 +628,6 @@ module ActionController #:nodoc:
       class Sweeper < ActiveRecord::Observer #:nodoc:
         attr_accessor :controller
 
-        # ActiveRecord::Observer will mark this class as reloadable even though it should not be.
-        # However, subclasses of ActionController::Caching::Sweeper should be Reloadable
-        include Reloadable::Subclasses
-        
         def before(controller)
           self.controller = controller
           callback(:before)
@@ -536,20 +639,45 @@ module ActionController #:nodoc:
           self.controller = nil
         end
 
+        protected
+          # gets the action cache path for the given options.
+          def action_path_for(options)
+            ActionController::Caching::Actions::ActionCachePath.path_for(controller, options)
+          end
+
+          # Retrieve instance variables set in the controller.
+          def assigns(key)
+            controller.instance_variable_get("@#{key}")
+          end
+
         private
           def callback(timing)
             controller_callback_method_name = "#{timing}_#{controller.controller_name.underscore}"
             action_callback_method_name     = "#{controller_callback_method_name}_#{controller.action_name}"
 
-            send(controller_callback_method_name) if respond_to?(controller_callback_method_name)
-            send(action_callback_method_name)     if respond_to?(action_callback_method_name)
+            send!(controller_callback_method_name) if respond_to?(controller_callback_method_name, true)
+            send!(action_callback_method_name)     if respond_to?(action_callback_method_name, true)
           end
 
           def method_missing(method, *arguments)
             return if @controller.nil?
-            @controller.send(method, *arguments)
+            @controller.send!(method, *arguments)
           end
       end
     end
+
+    module SqlCache
+      def self.included(base) #:nodoc:
+        if defined?(ActiveRecord) && ActiveRecord::Base.respond_to?(:cache)
+          base.alias_method_chain :perform_action, :caching
+        end
+      end
+
+      def perform_action_with_caching
+        ActiveRecord::Base.cache do
+          perform_action_without_caching
+        end
+      end
+    end
   end
 end
